@@ -21,13 +21,20 @@
    Author: Jaka Mocnik <jaka@gnu.org>
 */
 
-#include <config.h>
-#include <gnome.h>
+#ifdef HAVE_CONFIG_H
+#  include <config.h>
+#endif /* HAVE_CONFIG_H */
+
+#include <gtk/gtk.h>
 
 #include <libgnomeprintui/gnome-font-dialog.h>
 
-#include "ghex.h"
 #include "gnome-print-font-picker.h"
+#include "preferences.h"
+#include "configuration.h"
+#include "ghex-window.h"
+
+#define MAX_MAX_UNDO_DEPTH 100000
 
 static void select_font_cb(GtkWidget *w, const gchar *font_name,
 						   PropertyUI *pui);
@@ -39,15 +46,11 @@ static void prefs_response_cb(GtkDialog *dlg, gint response, PropertyUI *pui);
 static void offsets_col_cb(GtkToggleButton *tb, PropertyUI *pui);
 static void group_type_cb(GtkRadioButton *rd, PropertyUI *pui);
 static void format_activated_cb(GtkEntry *entry, PropertyUI *pui);
+static gboolean format_focus_out_event_cb(GtkEntry *entry, GdkEventFocus *event,
+										  PropertyUI *pui);
 
 PropertyUI *prefs_ui = NULL;
 
-PangoFontMetrics *def_metrics = NULL; /* Changes for Gnome 2.0 */
-PangoFontDescription *def_font_desc = NULL;
-
-gchar *def_font_name = NULL;
-gboolean show_offsets_column = TRUE;
-
 PropertyUI *
 create_prefs_dialog()
 {
@@ -129,6 +132,8 @@ create_prefs_dialog()
 	pui->format = gtk_entry_new();
 	g_signal_connect(G_OBJECT(pui->format), "activate",
 					 G_CALLBACK(format_activated_cb), pui);
+	g_signal_connect(G_OBJECT(pui->format), "focus_out_event",
+					 G_CALLBACK(format_focus_out_event_cb), pui);
 	gtk_box_pack_start (GTK_BOX(box), pui->format, TRUE, TRUE, GNOME_PAD);
 	gtk_widget_show(pui->format);
 
@@ -350,7 +355,7 @@ create_prefs_dialog()
 }
 
 
-void set_prefs(PropertyUI *pui) {
+void set_current_prefs(PropertyUI *pui) {
 	int i;
 
 	for(i = 0; i < 3; i++)
@@ -445,9 +450,24 @@ group_type_cb(GtkRadioButton *rd, PropertyUI *pui)
 static void
 prefs_response_cb(GtkDialog *dlg, gint response, PropertyUI *pui)
 {
+	GError *error = NULL;
+
 	switch(response) {
 	case GTK_RESPONSE_HELP:
-		/* TODO: display help */
+		gnome_help_display("ghex2.xml", "ghex-prefs", &error);
+		if(NULL != error) {
+			GtkWidget *dialog;
+			dialog = gtk_message_dialog_new
+				(NULL, GTK_DIALOG_MODAL, GTK_MESSAGE_ERROR, GTK_BUTTONS_CLOSE,
+				 _("There was an error displaying help: \n%s"),
+				 error->message);
+			g_signal_connect(G_OBJECT (dialog), "response",
+							 G_CALLBACK (gtk_widget_destroy),
+							 NULL);
+			gtk_window_set_resizable(GTK_WINDOW (dialog), FALSE);
+			gtk_widget_show(dialog);
+			g_error_free(error);
+		}
 		break;
 	case GTK_RESPONSE_CLOSE:
 		gtk_widget_hide(pui->pbox);
@@ -521,11 +541,12 @@ select_font_cb(GtkWidget *w, const gchar *font_name, PropertyUI *pui)
 }
 
 static void
-format_activated_cb(GtkEntry *entry, PropertyUI *pui)
+update_offset_fmt_from_entry(GtkEntry *entry, PropertyUI *pui)
 {
 	int i, len;
 	gchar *old_offset_fmt;
 	gboolean expect_spec;
+	const GList *win_list;
 
 	old_offset_fmt = offset_fmt;
 	offset_fmt = g_strdup(gtk_entry_get_text(GTK_ENTRY(pui->format)));
@@ -557,7 +578,7 @@ format_activated_cb(GtkEntry *entry, PropertyUI *pui)
 											 "Only 'x', 'X', 'p', 'P', 'd' and 'o' are allowed."));
 				gtk_dialog_run(GTK_DIALOG(msg_dialog));
 				gtk_widget_destroy(msg_dialog);
-				gtk_window_set_focus(GTK_WINDOW(pui->pbox), pui->format);
+				gtk_widget_grab_focus(pui->format);
 				break;
 			}
 		}
@@ -568,6 +589,24 @@ format_activated_cb(GtkEntry *entry, PropertyUI *pui)
 							 GHEX_BASE_KEY GHEX_PREF_OFFSET_FORMAT,
 							 offset_fmt,
 							 NULL);
+	win_list = ghex_window_get_list();
+	while(NULL != win_list) {
+		ghex_window_update_status_message((GHexWindow *)win_list->data);
+		win_list = win_list->next;
+	}
+}
+
+static gboolean
+format_focus_out_event_cb(GtkEntry *entry, GdkEventFocus *event, PropertyUI *pui)
+{
+	update_offset_fmt_from_entry(entry, pui);
+	return FALSE;
+}
+
+static void
+format_activated_cb(GtkEntry *entry, PropertyUI *pui)
+{
+	update_offset_fmt_from_entry(entry, pui);
 }
 
 static void
@@ -588,5 +627,5 @@ offset_cb(GtkWidget *w, PropertyUI *pui)
 		gtk_widget_set_sensitive(pui->format, TRUE);
 		break;
 	}
-	format_activated_cb(GTK_ENTRY(pui->format), pui);
+	update_offset_fmt_from_entry(GTK_ENTRY(pui->format), pui);
 }
